library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_signed.all;
use IEEE.STD_LOGIC_arith.all;

entity operazioni is
	 port(
		 Nw : in STD_LOGIC;
		 Clk : in STD_LOGIC;
		 Din : in STD_LOGIC_VECTOR(7 downto 0);
		 Ready : out STD_LOGIC;
		 Ris : out STD_LOGIC_VECTOR(15 downto 0)
	     );
end operazioni;

architecture beh of operazioni is

type stato is (IDLE, D0_LR, D1_LR, D0_HR, D1_HR, EXECUTE);
signal s: stato;
signal D0, D1: std_logic_vector(15 downto 0);
signal op: std_logic_vector(1 downto 0);
signal enD0LR, enD1LR, enD0HR, enD1HR, enexe: std_logic;

begin
	
	-- ** control unit ** --
	
	process(clk)
	begin
		if clk'event and clk = '0' then
			case s is
				when IDLE =>
					if nw = '1' then s <= D0_LR;
					else s <= IDLE;
					end if;
				when D0_LR =>
					s <= D1_LR;
				when D1_LR =>
					if op(0) = '0' then s <= EXECUTE;
					else s <= D0_HR;
					end if;
				when D0_HR =>
					s <= D1_HR;
				when D1_HR =>
					s <= EXECUTE;
				when EXECUTE =>
					s <= IDLE;
				when OTHERS =>
					s <= IDLE;
			end case;
		end if;
	end process;
	
	-- segnali di controllo
	 
	enD0LR <= '1' when s = D0_LR else '0'; 
	enD1LR <= '1' when s = D1_LR else '0'; 
	enD0HR <= '1' when s = D0_HR else '0'; 
	enD1HR <= '1' when s = D1_HR else '0'; 
	enexe <= '1' when s = EXECUTE else '0'; 
	
		
	
	
	-- ** datapath ** -- 
	
	process(clk) 
	variable tmp : std_logic_vector(15 downto 0); 
	begin 
		if clk'event and clk = '0' then	
			
			if nw = '1' then op <= Din(1 downto 0);
			end if;
			
			if enD0LR = '1' then 
				D0(7 downto 0) <= Din;	   
				-- estensione in segno del byte letto
				if din(7) = '1' then D0(15 downto 8) <= (others => '1');
								else D0(15 downto 8) <= (others => '0');
				end if;
			end if;
			
			if enD1LR = '1' then 
				D1(7 downto 0) <= Din;	 
				-- estensione in segno del byte letto
				if din(7) = '1' then D1(15 downto 8) <= (others => '1');
								else D1(15 downto 8) <= (others => '0');
				end if;
			end if;
			
			if enD0HR = '1' then D0(15 downto 8) <= Din;
			end if;
			
			if enD1HR = '1' then D1(15 downto 8) <= Din;
			end if;
			
			if enexe = '1' then 					-- **
				case op is 
				 when "00" => -- and 8	
								Ris <= d0 and d1;
				 when "01" => -- or 16
				 				Ris <= d0 or d1;
				 when "10" | "11" => -- add 8 e add16
				 				Ris <= d0 + d1;
				 when others => null;
				end case;
			end if;
			
			Ready <= enexe;
			
		end if;
	end process;
	

end beh;
